document.addEventListener('DOMContentLoaded', () => {
  const faAnims = document.querySelectorAll('.fa-hover');
  const productAccordion = document.getElementById('productsAccordion');
  const productAccordionItems = productAccordion.querySelectorAll('.accordion-button');
  const menuLinks = document.querySelectorAll('.menu-link');
  const navCollapseInstance = document.getElementById('navbar-content');
  const hamburgerToggle = document.querySelector('.hamburger-toggle');

  faAnims.forEach(item => {
    item.addEventListener('mouseover', () => toggleFaClass(item, true));
    item.addEventListener('mouseout', () => toggleFaClass(item, false));
  });

  menuLinks.forEach(menuItem => {
    ['mouseover', 'focus'].forEach(event => {
      menuItem.addEventListener(event, (e) => handleMenuLinkHover(e, menuItem));
    });
  });

  if (hamburgerToggle) {
    hamburgerToggle.addEventListener('click', toggleHamburgerMenu);
  }

  navCollapseInstance.addEventListener('click', (e) => {
    initializeScrollspy(e);
    setInitialScrollspyHeight(); 
  });

  productAccordion.addEventListener('shown.bs.collapse', (e) => {
    handleNavCollapseShown(e, productAccordionItems)
  });

  const searchOffCanvas = document.getElementById('offcanvasSearch');
  searchOffCanvas.addEventListener('shown.bs.offcanvas', () => {
    document.getElementById('searchInput').focus();
  });

  setActiveAccordion();
  setInitialScrollspyHeight();

  let resizeTimeout;
  window.addEventListener('resize', () => {
    clearTimeout(resizeTimeout);
    resizeTimeout = setTimeout(setActiveAccordion, 100);
  });

});

function setInitialScrollspyHeight() {
  let content = document.getElementById('mower-scrollspy')
  if (window.innerWidth > 768) {    ;
    if (typeof content != 'undefined' && content != null) {
      let height = document.getElementById('productsAccordion').offsetHeight;
      content.style.height = height + 'px';
    }
  } else {
    if (content) {
      content.style.removeProperty('height');
    }
  }  
}

function setActiveAccordion() {
  if (window.innerWidth < 769) {
    document.querySelectorAll('#navbar-content .accordion-collapse.show').forEach(element => {
      element.classList.remove('show');
    });
    document.querySelectorAll('#navbar-content .collapse.show').forEach(element => {
      element.classList.remove('show');
    });
    document.querySelectorAll('#navbar-content .accordion-button').forEach(element => {
      element.classList.add('collapsed');
    });
  } else {
    document.querySelectorAll('#navbar-content .accordion-collapse.show').forEach(element => {
      element.classList.remove('show');
    });    
    document.querySelectorAll('#navbar-content .collapse.show').forEach(element => {
      element.classList.remove('show');
    });    
    document.querySelectorAll('#navbar-content .accordion-button').forEach(element => {
      element.classList.add('collapsed');
    });
    document.getElementById('collapseMowers').classList.add('show');
    document.querySelector('#mowersHeading .accordion-button').classList.remove('collapsed');    
    document.querySelectorAll('#navbar-content .mower-collapse').forEach(element => {
      element.classList.add('show');
    });
      
  }
}

function toggleFaClass(item, add) {
  const className = item.dataset.faAnimation;
  if (add) {
    item.classList.add(className);
  } else {
    item.classList.remove(className);
  }
}

function toggleHamburgerMenu(e) {
  this.children[0].classList.toggle('active');
}

function initializeScrollspy(e) {
  const dataSpyList = e.currentTarget.querySelectorAll('.menu-scrollspy');
  dataSpyList.forEach(dataSpyEl => {
    let spy = new bootstrap.ScrollSpy(dataSpyEl, { 
      target: dataSpyEl.id,
      smoothScroll: true,
      rootMargin: '0px 0px -75%',
      threshold: [0.0]
    });
  });
}

function handleNavCollapseShown(e, navCollapseItems) {
  navCollapseItems.forEach(item => item.removeAttribute('tabindex'));
  let btn = e.target.previousElementSibling?.querySelector('button');
  if (btn) {
    btn.setAttribute('tabindex', -1);
    btn.blur();
  }
  //Logic to set the height of the scrollspy on accordion change.
  let content = e.target.querySelector('.menu-scrollspy');
  if (content != null) {
    new bootstrap.ScrollSpy(content, {
      target: content.id,
      smoothScroll: true,
      rootMargin: '0px 0px -40%',
      threshold: [0.0]
    })
  }
  if (window.innerWidth > 768) {
    if (typeof content != 'undefined' && content != null) {
      let height = document.getElementById('productsAccordion').offsetHeight;
      content.style.height = height+'px';
    }
  } else {
    if (content != null) {
      content.style.removeProperty('height');
    }    
  }
}

function handleMenuLinkHover(e, menuItem) {
  const parent = e.target.parentNode;
  const hoveredItem = parent.querySelector('.menu-link.hovered');
  if (hoveredItem) hoveredItem.classList.remove('hovered');

  const { imageId, hoverImage: image, imageAlt: altText } = menuItem.dataset;
  menuItem.classList.add('hovered');
  if (image) {
    const imgElement = document.getElementById(imageId);
    imgElement.src = image;
    imgElement.alt = altText;
  }
}
